==> recime-bot-runtime-2017.10.30938.tgz: package/src/slack.ts <==
import fs = require("fs");
import request = require("request");

import {Channel} from './channel';
import {Logger} from "./logger";

import db from "recime-keyvalue-store";

export class Slack extends Channel {
    private logger: any;
   
    constructor(params: any){
        super({
            homedir : params.homedir,
            uid : params.uid,
            platform : 'slack'
        });
    }

    preprocessParams(params){
        Object.keys(params).forEach(function(name) {
            var param = params[name]
            if (param &&
                typeof param === 'object') {
                params[name] = JSON.stringify(param)
            }
        });
        return params;
    }

    authorize(code:string){
        return new Promise((resolve, reject)=>{
            const payload = {
                url : "https://slack.com/api/oauth.access",
                method : "POST",
                form : this.preprocessParams({
                    code : code,
                    client_id : process.env.RECIME_SLACK_CLIENT_ID,
                    client_secret: process.env.RECIME_SLACK_CLIENT_SECRET,
                    redirect_uri : process.env.RECIME_SLACK_REDIRECT_URI
                })
            };
            request(payload, (err, response, body)=>{
                    const result = JSON.parse(body);
                    if (result.bot){
                        db.set(result.team_id, result.bot.bot_access_token).then((_)=>{
                            resolve({
                                success : true,
                                data : result
                            });
                        });
                    } else {
                        resolve(result);
                    }
            });
        });
    }

    getUser(data){
        let user = data.event.user;
        return new Promise((resolve, reject)=>{
            db.get(data.team_id).then ((token)=>{
                const methodName = "users.info";
                let payload = {
                    url: `https://slack.com/api/${methodName}`,
                    method: "POST",
                    form: this.preprocessParams({
                        token : token || process.env.SLACK_ACCESS_TOKEN || process.env.RECIME_SLACK_ACCESS_TOKEN,
                        user : user
                    })
                }
                request(payload, (err, response, body)=>{
                    const result = JSON.parse(body);
                    if (result.ok)
                        resolve(result.user);
                    else
                        resolve(user);
                });
            });
        });
    }

    postMessage(body, data){
        const channel = body.event.channel;
        const methodName= "chat.postMessage";
        return new Promise((resolve, reject)=>{
            db.get(body.team_id).then ((token)=>{
                let params = {
                    token : token || process.env.SLACK_ACCESS_TOKEN || process.env.RECIME_SLACK_ACCESS_TOKEN,
                    text : data.text || "_param.text_ is undefined",
                    channel: channel,
                    link_names : data.link_names,
                    attachments: data.attachments,
                    username : data.username,
                    as_user : data.as_user,
                    icon_url : data.icon_url,
                    icon_emoji : data.icon_emoji,
                    parse : data.parse,
                    unfurl_links : data.unfurl_links,
                    unfurl_media: data.unfurl_media,
                    thread_ts : data.thread_ts,
                    reply_broadcast: data.reply_broadcast
                };

                let payload = {
                    url: `https://slack.com/api/${methodName}`,
                    method: "POST",
                    form: this.preprocessParams(params)
                }

                request(payload, (err, response, body)=>{
                    if (err){
                        resolve(err);
                        return;
                    }
                    this.logger.outgoing(body);
                    resolve(body);
                });
            });
        });
    }

    execute(body, parameters, handler){
        return new Promise((resolve, reject)=>{
            Logger.init(this.params).then((logger)=>{
                if (body.event.subtype === 'bot_message'){
                    resolve({
                        success : true
                    });
                    return;
                }
                this.logger = logger;
                this.logger.incoming(body);

                this.getUser(body).then((user)=>{
                    body.event.user = user;
                    handler(this.params.uid, body, "slack", (data)=>{
                        if (data && data.text){
                            this.postMessage(body, data).then((result)=>{
                                resolve(result);
                            });
                        } else {
                            resolve();
                        }
                    }, (err)=>{
                        reject(err);
                    });
                });
            });
        });
    }

    resolveSuccessDialogPath(){
         let path = `${this.params.homedir}/${this.params.uid}/dialog.html`

         if (fs.existsSync(path)){
             return path;
         }
         return `${__dirname}/../slack-oauth-dialog.html`;
    }
    
    verifyToken(body){
        return new Promise((resolve, reject)=>{
            if (body.type === "url_verification"){
                resolve({
                    challenge : body.challenge
                });
            }else {
                reject("Invalid Access.");
            }
        });
    }
}

==> recime-bot-runtime-2017.10.260204.tgz: package/src/slack.ts <==
import fs = require("fs");
import request = require("request");

import {Channel} from './channel';
import {Logger} from "./logger";

import db from "recime-keyvalue-store";

export class Slack extends Channel {
    private logger: any;
   
    constructor(params: any){
        super({
            homedir : params.homedir,
            uid : params.uid,
            platform : 'slack'
        });
    }

    preprocessParams(params){
        Object.keys(params).forEach(function(name) {
            var param = params[name]
            if (param &&
                typeof param === 'object') {
                params[name] = JSON.stringify(param)
            }
        });
        return params;
    }

    authorize(code:string){
        return new Promise((resolve, reject)=>{
            const payload = {
                url : "https://slack.com/api/oauth.access",
                method : "POST",
                form : this.preprocessParams({
                    code : code,
                    client_id : process.env.RECIME_SLACK_CLIENT_ID,
                    client_secret: process.env.RECIME_SLACK_CLIENT_SECRET,
                    redirect_uri : process.env.RECIME_SLACK_REDIRECT_URI
                })
            };
            request(payload, (err, response, body)=>{
                    const result = JSON.parse(body);
                    if (result.bot){
                        const uid = `${this.params.uid}-${result.team_id}`;
                        db.set(uid, result.bot.bot_access_token).then((_)=>{
                            resolve({
                                success : true,
                                data : result
                            });
                        });
                    } else {
                        resolve(result);
                    }
            });
        });
    }

    getUser(data){
        let user = data.event.user;
        return new Promise((resolve, reject)=>{
            const uid = `${this.params.uid}-${data.team_id}`;
            db.get(uid).then ((token)=>{
                const methodName = "users.info";
                let payload = {
                    url: `https://slack.com/api/${methodName}`,
                    method: "POST",
                    form: this.preprocessParams({
                        token : token || process.env.SLACK_ACCESS_TOKEN || process.env.RECIME_SLACK_ACCESS_TOKEN,
                        user : user
                    })
                }
                request(payload, (err, response, body)=>{
                    const result = JSON.parse(body);
                    if (result.ok)
                        resolve(result.user);
                    else
                        resolve(user);
                });
            });
        });
    }

    postMessage(body, data){
        const channel = body.event.channel;
        const methodName= "chat.postMessage";
        return new Promise((resolve, reject)=>{
            const uid = `${this.params.uid}-${body.team_id}`;
            db.get(uid).then ((token)=>{
                let params = {
                    token : token || process.env.SLACK_ACCESS_TOKEN || process.env.RECIME_SLACK_ACCESS_TOKEN,
                    text : data.text || "_param.text_ is undefined",
                    channel: channel,
                    link_names : data.link_names,
                    attachments: data.attachments,
                    username : data.username,
                    as_user : data.as_user,
                    icon_url : data.icon_url,
                    icon_emoji : data.icon_emoji,
                    parse : data.parse,
                    unfurl_links : data.unfurl_links,
                    unfurl_media: data.unfurl_media,
                    thread_ts : data.thread_ts,
                    reply_broadcast: data.reply_broadcast
                };

                let payload = {
                    url: `https://slack.com/api/${methodName}`,
                    method: "POST",
                    form: this.preprocessParams(params)
                }

                request(payload, (err, response, body)=>{
                    if (err){
                        resolve(err);
                        return;
                    }
                    this.logger.outgoing(body);
                    resolve(body);
                });
            });
        });
    }

    execute(body, parameters, handler){
        return new Promise((resolve, reject)=>{
            Logger.init(this.params).then((logger)=>{
                if (body.event.subtype === 'bot_message'){
                    resolve({
                        success : true
                    });
                    return;
                }
                this.logger = logger;
                this.logger.incoming(body);

                this.getUser(body).then((user)=>{
                    body.event.user = user;
                    handler(this.params.uid, body, "slack", (data)=>{
                        if (data && data.text){
                            this.postMessage(body, data).then((result)=>{
                                resolve(result);
                            });
                        } else {
                            resolve();
                        }
                    }, (err)=>{
                        reject(err);
                    });
                });
            });
        });
    }

    resolveSuccessDialogPath(){
         const path = `${this.params.homedir}/${this.params.uid}/dialog.html`;
        
         if (fs.existsSync(path)){
             return path;
         }
         return `${__dirname}/../slack-oauth-dialog.html`;
    }
    
    verifyToken(body){
        return new Promise((resolve, reject)=>{
            if (body.type === "url_verification"){
                resolve({
                    challenge : body.challenge
                });
            }else {
                reject("Invalid Access.");
            }
        });
    }
}
